@@ -1,6 +1,6 @@
 /* -*- C++ -*-
  * File: dcraw_emu.cpp
- * Copyright 2008-2010 LibRaw LLC (info@libraw.org)
+ * Copyright 2008-2013 LibRaw LLC (info@libraw.org)
  * Created: Sun Mar 23,   2008
  *
  * LibRaw simple C++ API sample: almost complete dcraw emulator
@@ -32,17 +32,21 @@ it under the terms of the one of three licenses as you choose:
 #include <math.h>
 #include <ctype.h>
 #ifndef WIN32
-#include <fcntl.h>
-#include <unistd.h>
-#include <sys/stat.h>
 #include <sys/mman.h>
 #include <sys/time.h>
+#include <unistd.h>
+#else
+#include <io.h>
 #endif
+#include <fcntl.h>
+#include <sys/stat.h>
 
 #include "libraw/libraw.h"
 #ifdef WIN32
 #define snprintf _snprintf
 #include <windows.h>
+#else
+#define O_BINARY 0
 #endif
 
 
@@ -109,10 +113,18 @@ void usage(const char *prog)
 "-agreen <g> equilibrate green\n"
 #endif
 "-aexpo <e p> exposure correction\n"
+// WF
+"-dbnd <r g b g> debanding\n"
 #ifndef WIN32
 "-mmap     Use mmap()-ed buffer instead of plain FILE I/O\n"
 #endif
-        );
+"-mem	   Use memory buffer instead of FILE I/O\n"
+"-disars   Do not use RawSpeed library\n"
+"-disinterp Do not run interpolation step\n"
+"-dsrawrgb1 Disable YCbCr to RGB conversion for sRAW (Cb/Cr interpolation enabled)\n"
+"-dsrawrgb2 Disable YCbCr to RGB conversion for sRAW (Cb/Cr interpolation disabled)\n"
+"-disadcf  Do not use dcraw Foveon code either if compiled with demosaic-pack-GPL2\n"
+);
     exit(1);
 }
 
@@ -174,12 +186,12 @@ int main(int argc, char *argv[])
     LibRaw RawProcessor;
     int i,arg,c,ret;
     char opm,opt,*cp,*sp;
-    int use_bigfile=0, use_timing=0;
+    int use_bigfile=0, use_timing=0,use_mem=0;
 #ifndef WIN32
     int msize = 0,use_mmap=0;
-    void *iobuffer=0;
+    
 #endif
-
+	void *iobuffer=0;
 #ifdef OUT
 #undef OUT
 #endif
@@ -192,7 +204,7 @@ int main(int argc, char *argv[])
           char *optstr = argv[arg];
           opt = argv[arg++][1];
           if ((cp = strchr (sp=(char*)"cnbrkStqmHABCgU", opt))!=0)
-              for (i=0; i < "111411111142"[cp-sp]-'0'; i++)
+              for (i=0; i < "111411111144221"[cp-sp]-'0'; i++)
                   if (!isdigit(argv[arg+i][0]) && !optstr[2]) 
                       {
                           fprintf (stderr,"Non-numeric argument to \"-%c\"\n", opt);
@@ -239,6 +251,9 @@ int main(int argc, char *argv[])
                       use_mmap              = 1;
                   else
 #endif
+				  if(!strcmp(optstr,"-mem"))
+                      use_mem              = 1;
+                  else
                       {
                           if(!argv[arg-1][2])
                               OUT.med_passes  = atoi(argv[arg++]);  
@@ -258,8 +273,9 @@ int main(int argc, char *argv[])
               case 'p':  OUT.camera_profile = argv[arg++];
 #endif
                   break;
-              case 'h':  OUT.half_size         = 1;		
-                  // no break:  "-h" implies "-f" 
+              case 'h':  
+                  OUT.half_size         = 1;		
+                  break;
               case 'f':  
                   if(!strcmp(optstr,"-fbdd"))
                       OUT.fbdd_noiserd = atoi(argv[arg++]);
@@ -312,7 +328,7 @@ int main(int argc, char *argv[])
                               fprintf (stderr,"Unknown option \"%s\".\n",argv[arg-1]);
                   break;
               case 'w':  OUT.use_camera_wb     = 1;  break;
-              case 'M':  OUT.use_camera_matrix = (opm == '+');  break;
+              case 'M':  OUT.use_camera_matrix = (opm == '+'); break;
               case 'j':  OUT.use_fuji_rotate   = 0;  break;
               case 'W':  OUT.no_auto_bright    = 1;  break;
               case 'T':  OUT.output_tiff       = 1;  break;
@@ -321,9 +337,25 @@ int main(int argc, char *argv[])
               case 'F':  use_bigfile=1; break;
               case 'd':
                   if(!strcmp(optstr,"-dcbi"))
-                      OUT.dcb_iterations = atoi(argv[arg++]);
+                    OUT.dcb_iterations = atoi(argv[arg++]);
+                  else if(!strcmp(optstr,"-disars"))
+                    OUT.use_rawspeed=0;
+                  else if(!strcmp(optstr,"-disadcf"))
+                    OUT.force_foveon_x3f=1;
+                  else if(!strcmp(optstr,"-disinterp"))
+                    OUT.no_interpolation=1;
                   else if(!strcmp(optstr,"-dcbe"))
                       OUT.dcb_enhance_fl = 1;
+                  else if(!strcmp(optstr,"-dsrawrgb1"))
+                      OUT.sraw_ycc = 1;
+                  else if(!strcmp(optstr,"-dsrawrgb2"))
+                      OUT.sraw_ycc = 2;
+                  else if(!strcmp(optstr,"-dbnd"))
+                  {
+                  	for(c=0; c<4; c++)
+                            OUT.wf_deband_treshold[c] = (float)atof(argv[arg++]);
+					OUT.wf_debanding = 1;
+                  }
                   else
                       fprintf (stderr,"Unknown option \"%s\".\n",argv[arg-1]);
                   break;
@@ -347,7 +379,6 @@ int main(int argc, char *argv[])
 #else
   _putenv ((char*)"TZ=UTC"); // dcraw compatibility, affects TIFF datestamp field
 #endif
-  OUT.filtering_mode = LIBRAW_FILTERING_AUTOMATIC;
 #define P1 RawProcessor.imgdata.idata
 #define S RawProcessor.imgdata.sizes
 #define C RawProcessor.imgdata.color
@@ -404,6 +435,44 @@ int main(int argc, char *argv[])
                 }
             else
 #endif
+			if (use_mem)
+			{
+				int file = open(argv[arg],O_RDONLY|O_BINARY);
+                    struct stat st;
+                    if(file<0)
+                        {
+                            fprintf(stderr,"Cannot open %s: %s\n",argv[arg],strerror(errno));
+                            continue;
+                        }
+                    if(fstat(file,&st))
+                        {
+                            fprintf(stderr,"Cannot stat %s: %s\n",argv[arg],strerror(errno));
+                            close(file);
+                            continue;
+                        }
+					if(!(iobuffer = malloc(st.st_size)))
+					{
+                                          fprintf(stderr,"Cannot allocate %d kbytes for memory buffer\n",(int)(st.st_size/1024));
+						close(file);
+						continue;
+					}
+					int rd;
+					if(st.st_size!=(rd=read(file,iobuffer,st.st_size)))
+					{
+                                          fprintf(stderr,"Cannot read %d bytes instead of  %d to memory buffer\n",(int)rd,(int)st.st_size);
+						close(file);
+						free(iobuffer);
+						continue;
+					}
+					close(file);
+					if( (ret = RawProcessor.open_buffer(iobuffer,st.st_size) != LIBRAW_SUCCESS))
+                    {
+                            fprintf(stderr,"Cannot open_buffer %s: %s\n",argv[arg],libraw_strerror(ret));
+							free(iobuffer);
+                            continue; // no recycle b/c open file will recycle itself
+                    }
+			}
+			else
                 {
                     if(use_bigfile)
                         // force open_file switch to bigfile processing
@@ -461,6 +530,11 @@ int main(int argc, char *argv[])
                     iobuffer=0;
                 }
 #endif
+			else if(use_mem && iobuffer)
+			{
+				free(iobuffer);
+				iobuffer = 0;
+			}
             
             RawProcessor.recycle(); // just for show this call
         }
